#include "initgl.h"
#include <GL/glx.h>
#include <GL/gl.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/keysym.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "textures.h"
#include "mathlib.h"
#include "objets.h"

GLuint BlurTexture;

void ViewOrtho(void)
{
  unsigned int width,height;
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  getScreenSize(&width,&height);
  glOrtho( 0, width , height , 0, -1, 1 );
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
}

void ViewPerspective()
{
  glMatrixMode( GL_PROJECTION );
  glPopMatrix();
  glMatrixMode( GL_MODELVIEW );
  glPopMatrix();
}

void RenderToTexture()
{
  unsigned int width,height;
  glViewport(0,0,128,128);
  
  tetraedreAfficher();

  glBindTexture(GL_TEXTURE_2D,BlurTexture);  
  glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, 0, 0, 128, 128, 0);

  glClearColor(0.0f, 0.0f, 0.5f, 0.5);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  getScreenSize(&width,&height);
  glViewport(0 , 0,width,height);
}

void DrawBlur(int times, float inc)
{
  unsigned int width,height;
  int num;
  float spost = 0.0f;
  float alphainc = 0.9f / times;
  float alpha = 0.2f;
  
  getScreenSize(&width,&height);

  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);
  
  glEnable(GL_TEXTURE_2D);
  glDisable(GL_DEPTH_TEST);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE);
  glEnable(GL_BLEND);
  glBindTexture(GL_TEXTURE_2D,BlurTexture);
  ViewOrtho();
  
  alphainc = alpha / times;
  
  glBegin(GL_QUADS);
  for (num = 0;num < times;num++)
    {
      glColor4f(1.0f, 1.0f, 1.0f, alpha);
      glTexCoord2f(0+spost,1-spost);
      glVertex2f(0,0);
      
      glTexCoord2f(0+spost,0+spost);
      glVertex2f(0,height);
      
      glTexCoord2f(1-spost,0+spost);
      glVertex2f(width,height);
      
      glTexCoord2f(1-spost,1-spost);
      glVertex2f(width,0);
      
      spost += inc;
      alpha = alpha - alphainc;
    }
  glEnd();
  
  ViewPerspective();
  
  glEnable(GL_DEPTH_TEST);
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);
  glBindTexture(GL_TEXTURE_2D,0);
}

BOOL Initialize(void)
{
  tetraedreInitialiser();

  glLoadIdentity();
  GLfloat global_ambient[4]={0.2f, 0.2f,  0.2f, 1.0f};
  GLfloat light0pos[4]=     {0.0f, 0.0f, -10.0f, 1.0f};
  GLfloat light0ambient[4]= {0.2f, 0.2f,  0.2f, 1.0f};
  GLfloat light0diffuse[4]= {0.3f, 0.3f,  0.3f, 1.0f};
  GLfloat light0specular[4]={0.8f, 0.8f,  0.8f, 1.0f};
  
  GLfloat lmodel_ambient[]= {0.2f,0.2f,0.2f,1.0f};
  
  BlurTexture = EmptyTexture();
  
  glLoadIdentity();
  
  glEnable(GL_DEPTH_TEST);
  
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT,lmodel_ambient);
  
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);
  glLightfv(GL_LIGHT0, GL_POSITION, light0pos);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light0ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light0diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light0specular);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  
  glShadeModel(GL_SMOOTH);
  
  glMateriali(GL_FRONT, GL_SHININESS, 128);
  glClearColor(0.0f, 0.0f, 0.0f, 0.5);
  
  return True;
}

void Deinitialize (void)
{
  glDeleteTextures(1,&BlurTexture);
}

void update (GLfloat elapsed)
{
}


void drawGLScene(void)
{
  glClearColor(0.0f, 0.0f, 0.0f, 0.5);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();
  RenderToTexture();
  tetraedreAfficher();
  tetraedreUpdater();
  DrawBlur(30.0,0.02f);
  glFlush();
}

int main(void)
{
  createGLWindow("alt3's render2texture",640,480,24,False);
  run();
  killGLWindow();
  
  return 0;
}
